/**
 * Copyright (c) 2007-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.PetalsAdminServiceClient;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * JMX client side Implementation of the JMX Petals Administration service.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class PetalsAdminServiceClientImpl extends AbstractServiceClient implements
        PetalsAdminServiceClient {
    
    /**
     * Action : stop the container
     */
    public static final String STOP = "stopContainer";

    /**
     * Action : shutdown the container
     */
    public static final String SHUTDOWN = "shutdownContainer";

    /**
     * Action : retrieve topology
     */
    public static final String RETRIEVE_SERVER_PROPERTIES = "retrieveServerProperties";
    
    /**
     * Action : ping
     */
    public static final String PING = "ping";

    /**
     * Creates a new instance of {@link PetalsAdminServiceClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @param isPetalsServer20Older
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     */
    public PetalsAdminServiceClientImpl(final String petalsDomain,
            final MBeanServerConnection beanServerConnection)
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, PetalsAdminServiceClient.PETALS_ADMIN_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new PetalsAdminDoesNotExistException();
            }
        } catch (IOException e) {
            throw new PetalsAdminServiceErrorException(e);
        } catch (MalformedObjectNameException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void stopContainer() throws PetalsAdminServiceErrorException {
        try {
            this.performAction(STOP, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void shutdownContainer() throws PetalsAdminServiceErrorException {
        try {
            this.performAction(SHUTDOWN, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Properties retrieveServerProperties() throws PetalsAdminServiceErrorException {
        try {
            return (Properties) this.performAction(RETRIEVE_SERVER_PROPERTIES, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean ping() throws PetalsAdminServiceErrorException {
        try {
            return ((Boolean) this.performAction(PING, null, null)).booleanValue();
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }
}
