/**
 * Copyright (c) 2009-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.configuration.component;

import java.io.IOException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.ow2.petals.jmx.api.api.configuration.component.AbstractConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentErrorException;
import org.ow2.petals.jmx.api.api.configuration.component.exception.RuntimeConfigurationComponentErrorException;
import org.ow2.petals.jmx.api.impl.AbstractServiceClient;

/**
 * 
 * 
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class AbstractConfigurationComponentClientImpl extends AbstractServiceClient
        implements AbstractConfigurationComponentClient {

    public AbstractConfigurationComponentClientImpl(final String petalsDomain,
            final MBeanServerConnection beanServerConnection, final String configurationMBeanName,
            final String configurationMBeanType)
            throws ConfigurationComponentDoesNotExistException, ConfigurationComponentErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, configurationMBeanName);
            attributes.put(JMX_FIELD_TYPE, configurationMBeanType);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new ConfigurationComponentDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new ConfigurationComponentErrorException(e);
        } catch (final IOException e) {
            throw new ConfigurationComponentErrorException(e);
        }
    }

    public AbstractConfigurationComponentClientImpl(final ObjectName objectName, final String petalsDomain,
            final MBeanServerConnection beanServerConnection) {
        super(petalsDomain, beanServerConnection);

        this.mbeanName = objectName;
    }

    @Override
    public Map<MBeanAttributeInfo, Object> getConfigurationMBeanAttributes() throws RuntimeConfigurationComponentErrorException {
        try {
            final Map<MBeanAttributeInfo, Object> attributes = new HashMap<MBeanAttributeInfo, Object>();
            final MBeanAttributeInfo[] attrs = this.mBeanServerConnection.getMBeanInfo(this.mbeanName).getAttributes();
            for (final MBeanAttributeInfo info : attrs) {
                attributes.put(info, this.mBeanServerConnection.getAttribute(this.mbeanName, info.getName()));
            }
            return attributes;

        } catch (final InstanceNotFoundException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        } catch (final IntrospectionException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        } catch (final ReflectionException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        } catch (final IOException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        } catch (final AttributeNotFoundException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        } catch (final MBeanException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        }
    }

    @Override
    public void setConfigurationMBeanAttributes(final Map<MBeanAttributeInfo, Object> attributes)
            throws RuntimeConfigurationComponentErrorException {
        final AttributeList attributesList = new AttributeList();
        for (final Entry<MBeanAttributeInfo, Object> entry : attributes.entrySet()) {
            attributesList.add(new Attribute(entry.getKey().getName(), entry.getValue()));
        }
        try {
            this.mBeanServerConnection.setAttributes(this.mbeanName, attributesList);

        } catch (final InstanceNotFoundException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        } catch (final ReflectionException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        } catch (final IOException e) {
            throw new RuntimeConfigurationComponentErrorException(e);
        }
    }
}
