/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import org.junit.Rule;
import org.junit.rules.RuleChain;
import org.junit.rules.TestRule;

/**
 * Abstract class for unit tests of the JMX API implementation to check that JMX client interfaces are aligned with the
 * JMX server MBean part.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractServiceClientImpl {

    /**
     * Port of the JMX server
     */
    private final static Integer JMX_PORT = Integer.valueOf(7070);

    /**
     * JBI identifier of a component
     */
    public static final String COMPONENT_ID = "my-component";

    /**
     * JBI identifier of a Petals BC SOAP
     */
    public static final String PETALS_BC_SOAP_ID = "petals-bc-soap";

    public final EmbeddedJmxServerConnector embeddedJmxSrvCon = new EmbeddedJmxServerConnector(JMX_PORT.intValue());
    
    public final EmbeddedJmxClient jmxClient = new EmbeddedJmxClient(JMX_PORT.intValue());

    @Rule
    public TestRule jmxTestRuleChain = RuleChain.outerRule(embeddedJmxSrvCon).around(jmxClient);
}
