/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.clientserverapi.jbi.messaging.endpoint.exception.EndpointDirectoryException;
import org.ow2.petals.jmx.api.api.EndpointDirectoryClient;
import org.ow2.petals.jmx.api.impl.mbean.EndpointDirectoryService;
import org.ow2.petals.jmx.api.impl.mbean.EndpointDirectoryServiceMBean;

/**
 * <p>
 * Unit tests of {@link EndpointDirectoryClientImpl} to check that JMX client interfaces are aligned with the JMX server
 * MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link EndpointDirectoryServiceMBean} to avoid to forget some methods. A unit test will only invoke
 * the associated method of {@link EndpointDirectoryServiceMBean}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class EndpointDirectoryClientImplTest extends AbstractServiceClientImpl implements EndpointDirectoryServiceMBean {

    private EndpointDirectoryClient endpointDirectoryClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerRegistryService(new EndpointDirectoryService());
        this.endpointDirectoryClient = jmxClient.getEndpointDirectoryClient();
    }

    /**
     * Check the interface consistency of {@link EndpointDirectoryClientImpl#getAllInternalEndpoints()}
     */
    @Test
    public void testGetAllInternalEndpoints() throws EndpointDirectoryException {
        this.getAllInternalEndpoints();
    }

    @Override
    public List<Map<String, Object>> getAllInternalEndpoints() throws EndpointDirectoryException {
        return this.endpointDirectoryClient.getAllInternalEndpoints();
    }

    /**
     * Check the interface consistency of {@link EndpointDirectoryClientImpl#getAllExternalEndpoints()}
     */
    @Test
    public void testGetAllExternalEndpoints() throws EndpointDirectoryException {
        this.getAllExternalEndpoints();
    }

    @Override
    public List<Map<String, Object>> getAllExternalEndpoints() throws EndpointDirectoryException {
        return this.endpointDirectoryClient.getAllExternalEndpoints();
    }

    /**
     * Check the interface consistency of {@link EndpointDirectoryClientImpl#getAllEndpoints()}
     */
    @Test
    public void testGetAllEndpoints() throws EndpointDirectoryException {
        this.getAllEndpoints();
    }

    @Override
    public List<Map<String, Object>> getAllEndpoints() throws EndpointDirectoryException {
        return this.endpointDirectoryClient.getAllEndpoints();
    }

    /**
     * Check the interface consistency of {@link EndpointDirectoryClientImpl#getDescription(String, String)}
     */
    @Test
    public void testGetDescription() throws EndpointDirectoryException {

        this.getDescription("serviceName", "endpointName");

    }

    @Override
    public String getDescription(final String serviceName, final String endpointName) throws EndpointDirectoryException {
        return this.endpointDirectoryClient.getDescription(serviceName, endpointName);
    }

}
