/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.net.InetAddress;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.junit.Test;
import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.EndpointDirectoryClient;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.LoggerServiceClient;
import org.ow2.petals.jmx.api.api.PetalsAdminServiceClient;
import org.ow2.petals.jmx.api.api.SystemMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.LocalTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.TcpTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectFilter;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectListener;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectEmitterDoesNotExistsException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectListenerException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.NoDefectListenerException;
import org.ow2.petals.jmx.api.impl.mbean.AdminServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.DeploymentServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.EndpointDirectoryServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.InstallationServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.LoggingServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.PetalsAdminServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.TopologyServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.monitoring.container.transport.local.LocalTransporterMonitoringServiceMBean;
import org.ow2.petals.jmx.api.impl.mbean.monitoring.container.transport.remote.TcpTransporterMonitoringServiceMBean;

/**
 * <p>
 * Unit tests about {@link JMXClientImpl} when the remote MBeans does not exist (are not registered).
 * </p>
 * <p>
 * This class implements {@link JMXClient} to avoid to forget to implement the unit test of an unexisting MBean.
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class JMXClientImplTest extends AbstractServiceClientImpl implements JMXClient {

    @Override
    public InetAddress getHost() throws NotRemoteJMXClientException {
        return null;
    }

    @Override
    public int getPort() throws NotRemoteJMXClientException {
        return 0;
    }

    @Override
    public String getUsername() throws NotRemoteJMXClientException {
        return null;
    }

    @Override
    public String getPassword() throws NotRemoteJMXClientException {
        return null;
    }

    @Override
    public void disconnect() throws ConnectionErrorException {
        // NOP
    }

    @Override
    public void subscribeToComponentMonitoringService(final String componentIdentifier,
            final DefectListener defectListener, DefectFilter defectFilter)
                    throws DefectListenerException, DefectEmitterDoesNotExistsException {
        // TODO Auto-generated method stub
    }

    @Override
    public void unsubscribeToComponentMonitoringService(final String componentIdentifier)
            throws NoDefectListenerException, DefectEmitterDoesNotExistsException, DefectListenerException {
        // TODO Auto-generated method stub
    }

    /**
     * {@link PetalsAdminServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = PetalsAdminDoesNotExistException.class)
    public void unexistingPetalsAdminServiceMBean()
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException, ConnectionErrorException {
        this.getPetalsAdminServiceClient();
    }

    @Override
    public PetalsAdminServiceClient getPetalsAdminServiceClient()
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException, ConnectionErrorException {
        return this.jmxClient.getPetalsAdminServiceClient();
    }

    /**
     * {@link AdminServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = AdminDoesNotExistException.class)
    public void unexistingAdminServiceMBean()
            throws AdminDoesNotExistException, AdminServiceErrorException, ConnectionErrorException {
        this.getAdminServiceClient();
    }

    @Override
    public AdminServiceClient getAdminServiceClient()
            throws AdminDoesNotExistException, AdminServiceErrorException, ConnectionErrorException {
        return this.jmxClient.getAdminServiceClient();
    }

    /**
     * {@link InstallationServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = InstallationServiceDoesNotExistException.class)
    public void unexistingInstallationServiceMBeant() throws InstallationServiceDoesNotExistException,
            InstallationServiceErrorException, ConnectionErrorException {
        this.getInstallationServiceClient();
    }

    @Override
    public InstallationServiceClient getInstallationServiceClient() throws InstallationServiceDoesNotExistException,
            InstallationServiceErrorException, ConnectionErrorException {
        return this.jmxClient.getInstallationServiceClient();
    }

    /**
     * {@link DeploymentServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = DeploymentServiceDoesNotExistException.class)
    public void unexistingDeploymentServiceMBean()
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException, ConnectionErrorException {
        this.getDeploymentServiceClient();
    }

    @Override
    public DeploymentServiceClient getDeploymentServiceClient()
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException, ConnectionErrorException {
        return this.jmxClient.getDeploymentServiceClient();
    }

    /**
     * The component lifecycle-MBean does not exist on the JMX server part.
     */
    @Test(expected = ComponentDoesNotExistException.class)
    public void unexistingComponentMBeanByName()
            throws ComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        this.getComponentClient("unexisting-component");
    }

    @Override
    public ComponentClient getComponentClient(final String componentName)
            throws ComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        return this.jmxClient.getComponentClient(componentName);
    }

    @Test(expected = ComponentDoesNotExistException.class)
    public void unexistingComponentMBeanByObjectNameNotInDomainPetals() throws ComponentDoesNotExistException,
            ComponentErrorException, ConnectionErrorException, MalformedObjectNameException {
        this.getComponentClient(new ObjectName("a-domain", "name", "unexisting-component"));
    }

    @Test
    public void unexistingComponentMBeanByObjectNameInDomainPetals() throws ComponentDoesNotExistException,
            ComponentErrorException, ConnectionErrorException, MalformedObjectNameException {
        this.getComponentClient(new ObjectName(JMXClientImpl.OW2_PETALS_DOMAIN, "name", "unexisting-component"));
    }

    @Override
    public ComponentClient getComponentClient(final ObjectName componentMBeanName)
            throws ComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        return this.jmxClient.getComponentClient(componentMBeanName);
    }

    /**
     * The component installer-MBean does not exist on the JMX server part.
     */
    @Test(expected = InstallerComponentDoesNotExistException.class)
    public void unexistingInstallerComponentMBean()
            throws InstallerComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        this.getInstallerComponentClient("unexisting-component");
    }

    @Override
    public InstallerComponentClient getInstallerComponentClient(final String componentName)
            throws InstallerComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException {
        return this.jmxClient.getInstallerComponentClient(componentName);
    }

    /**
     * {@link EndpointDirectoryServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = EndpointDirectoryServiceDoesNotExistException.class)
    public void unexistingEndpointDirectoryMBean() throws ConnectionErrorException,
            EndpointDirectoryServiceDoesNotExistException, EndpointDirectoryServiceErrorException {
        this.getEndpointDirectoryClient();
    }

    @Override
    public EndpointDirectoryClient getEndpointDirectoryClient() throws ConnectionErrorException,
            EndpointDirectoryServiceDoesNotExistException, EndpointDirectoryServiceErrorException {
        return this.jmxClient.getEndpointDirectoryClient();
    }

    /**
     * As the JVM includes always system MBeans, we can't check the processing when they does not exist. TODO: Find a
     * way to unregister these system MBean
     */
    @Test
    public void unexistingSystemMonitoringServiceMBean() throws ConnectionErrorException,
            SystemMonitoringDoesNotExistException, SystemMonitoringServiceErrorException {
        this.getSystemMonitoringServiceClient();
    }

    @Override
    public SystemMonitoringServiceClient getSystemMonitoringServiceClient() throws ConnectionErrorException,
            SystemMonitoringDoesNotExistException, SystemMonitoringServiceErrorException {
        return this.jmxClient.getSystemMonitoringServiceClient();
    }

    /**
     * {@link LoggingServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = LoggerServiceDoesNotExistException.class)
    public void unexistingLoggerServiceMBean()
            throws LoggerServiceErrorException, LoggerServiceDoesNotExistException, ConnectionErrorException {
        this.getLoggerServiceClient();
    }

    @Override
    public LoggerServiceClient getLoggerServiceClient()
            throws LoggerServiceErrorException, LoggerServiceDoesNotExistException, ConnectionErrorException {
        return this.jmxClient.getLoggerServiceClient();
    }

    /**
     * {@link LocalTransporterMonitoringServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = LocalTransporterMonitoringServiceDoesNotExistException.class)
    public void unexistingLocalTransporterMonitoringServiceMBean()
            throws LocalTransporterMonitoringServiceErrorException,
            LocalTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException {
        this.getLocalTransporterMonitoringServiceClient();
    }

    @Override
    public LocalTransporterMonitoringServiceClient getLocalTransporterMonitoringServiceClient()
            throws LocalTransporterMonitoringServiceErrorException,
            LocalTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException {
        return this.jmxClient.getLocalTransporterMonitoringServiceClient();
    }

    /**
     * {@link TcpTransporterMonitoringServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = TcpTransporterMonitoringServiceDoesNotExistException.class)
    public void unexistingTcpTransporterMonitoringServiceMBean() throws TcpTransporterMonitoringServiceErrorException,
            TcpTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException {
        this.getTcpTransporterMonitoringServiceClient();
    }

    @Override
    public TcpTransporterMonitoringServiceClient getTcpTransporterMonitoringServiceClient()
            throws TcpTransporterMonitoringServiceErrorException, TcpTransporterMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        return this.jmxClient.getTcpTransporterMonitoringServiceClient();
    }

    /**
     * The component monitoring-MBean does not exist on the JMX server part.
     */
    @Test(expected = ComponentMonitoringServiceDoesNotExistException.class)
    public void unexistingComponentMonitoringServiceMBean() throws ConnectionErrorException,
            ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException {
        this.getComponentMonitoringServiceClient("unexisting-component");
    }

    @Override
    public ComponentMonitoringServiceClient getComponentMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        return this.jmxClient.getComponentMonitoringServiceClient(componentIdentifier);
    }

    /**
     * The Petals BC SOAP monitoring-MBean does not exist on the JMX server part.
     */
    @Test(expected = ComponentMonitoringServiceDoesNotExistException.class)
    public void unexistingPetalsBcSoapMonitoringServiceMBean() throws ConnectionErrorException,
            ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException {
        this.getPetalsBcSoapMonitoringServiceClient("unexisting-bc-soap");
    }

    @Override
    public PetalsBcSoapMonitoringServiceClient getPetalsBcSoapMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        return this.jmxClient.getPetalsBcSoapMonitoringServiceClient(componentIdentifier);
    }

    /**
     * {@link TopologyServiceMBean} does not exist on the JMX server part.
     */
    @Test(expected = TopologyServiceDoesNotExistException.class)
    public void unexistingTopologyServiceMBean()
            throws TopologyServiceErrorException, TopologyServiceDoesNotExistException, ConnectionErrorException {
        this.getTopologyServiceClient();
    }

    @Override
    public TopologyServiceClient getTopologyServiceClient()
            throws TopologyServiceErrorException, TopologyServiceDoesNotExistException, ConnectionErrorException {
        return this.jmxClient.getTopologyServiceClient();
    }

}
