/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.configuration.component;

import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanAttributeInfo;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.configuration.component.AbstractConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.impl.AbstractServiceClientImpl;

/**
 * <p>
 * Abstract class for unit tests: {@link RuntimeConfigurationComponentClientImplTest} and
 * {@link InstallationConfigurationComponentClientImplTest}.
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractConfigurationComponentClientImplTest extends AbstractServiceClientImpl
        implements AbstractConfigurationComponentClient {

    protected AbstractConfigurationComponentClient abstractConfigurationComponentClient;

    @Before
    public void before() throws Exception {
        this.setJMXServerEnv(COMPONENT_ID);
        this.abstractConfigurationComponentClient = this
                .getComponentConfigurationServiceClient(COMPONENT_ID);
    }

    protected abstract void setJMXServerEnv(final String componentId) throws Exception;

    protected abstract AbstractConfigurationComponentClient getComponentConfigurationServiceClient(
            final String componentId) throws Exception;

    /**
     * Check the interface consistency of
     * {@link RuntimeConfigurationComponentClientImpl#getConfigurationMBeanAttributes()}
     */
    @Test
    public void testGetConfigurationMBeanAttributes() throws ConfigurationComponentErrorException {

        this.getConfigurationMBeanAttributes();
    }

    @Override
    public Map<MBeanAttributeInfo, Object> getConfigurationMBeanAttributes()
            throws ConfigurationComponentErrorException {

        return this.abstractConfigurationComponentClient.getConfigurationMBeanAttributes();
    }

    /**
     * Check the interface consistency of {@link RuntimeConfigurationComponentClientImpl#setAttributes(Map)}
     */
    @Test
    public void testSetAttributes() throws ConfigurationComponentErrorException {

        this.setConfigurationMBeanAttributes(new HashMap<MBeanAttributeInfo, Object>());
    }

    @Override
    public void setConfigurationMBeanAttributes(final Map<MBeanAttributeInfo, Object> attributes)
            throws ConfigurationComponentErrorException {

        this.abstractConfigurationComponentClient.setConfigurationMBeanAttributes(attributes);
    }

    /**
     * Check the interface consistency of {@link RuntimeConfigurationComponentClientImpl#getAttribute(String)}
     */
    @Test
    public void testGetAttribute() throws AttributeErrorException {

        this.getAttribute("an-attribute");
    }

    @Override
    public Object getAttribute(final String name) throws AttributeErrorException {
        return this.abstractConfigurationComponentClient.getAttribute(name);
    }

    /**
     * Check the interface consistency of {@link RuntimeConfigurationComponentClientImpl#setAttribute(String, Object)}
     */
    @Test
    public void testSetAttribute() throws AttributeErrorException {

        this.setAttribute("an-attribute", "a-value");
    }

    @Override
    public void setAttribute(final String name, final Object value) throws AttributeErrorException {
        this.abstractConfigurationComponentClient.setAttribute(name, value);
    }

}
