/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.configuration.component;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.configuration.component.AbstractConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.InstallationConfigurationComponentClient;
import org.ow2.petals.jmx.api.impl.mbean.Installer;
import org.ow2.petals.jmx.api.impl.mbean.InstallerResults;
import org.ow2.petals.jmx.api.impl.mbean.configuration.component.InstallationConfigurationComponentService;

/**
 * <p>
 * Unit tests of {@link InstallationConfigurationComponentClientImpl} to check that JMX client interfaces are aligned
 * with the JMX server MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link InstallationConfigurationComponentClient} to avoid to forget some methods. A unit
 * test will only invoke the associated method of {@link InstallationConfigurationComponentClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class InstallationConfigurationComponentClientImplTest extends AbstractConfigurationComponentClientImplTest
        implements InstallationConfigurationComponentClient {

    @Override
    protected void setJMXServerEnv(final String componentId) throws Exception {
        this.embeddedJmxSrvCon
                .registerInstallerComponentService(this.createComponentInstallerService(componentId), componentId)
                .registerConfigurationInstallerComponentService(this.createComponentInstallationConfigurationService(),
                        componentId);
    }

    protected Installer createComponentInstallerService(final String componentId) throws MalformedObjectNameException {
        return new Installer(new InstallerResults().setGetInstallerConfigurationMBeanResult(
                new ObjectName("org.ow2.petals:type=custom,name=bootstrap_" + componentId)));
    }

    protected InstallationConfigurationComponentService createComponentInstallationConfigurationService() {
        return new InstallationConfigurationComponentService();
    }

    @Override
    protected AbstractConfigurationComponentClient getComponentConfigurationServiceClient(final String componentId)
            throws Exception {
        return this.jmxClient.getInstallerComponentClient(componentId).getInstallationConfigurationClient();
    }

}
