/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.monitoring.component.soap;

import java.util.Map;

import org.junit.Test;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.IncomingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.OutgoingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.WsClientPoolKey;
import org.ow2.petals.jmx.api.api.monitoring.object.ThreadPoolMetrics;
import org.ow2.petals.jmx.api.impl.mbean.monitoring.component.framework.ComponentMonitoringService;
import org.ow2.petals.jmx.api.impl.mbean.monitoring.component.soap.PetalsBcSoapMonitoringService;
import org.ow2.petals.jmx.api.impl.monitoring.component.framework.ComponentMonitoringServiceClientImplTest;

/**
 * <p>
 * Unit tests of {@link PetalsBcSoapMonitoringServiceClientImpl} to check that JMX client interfaces are aligned with
 * the JMX server MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link PetalsBcSoapMonitoringServiceClient} to avoid to forget some methods. A unit test
 * will only invoke the associated method of {@link PetalsBcSoapMonitoringServiceClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PetalsBcSoapMonitoringServiceClientImplTest extends ComponentMonitoringServiceClientImplTest implements
        PetalsBcSoapMonitoringServiceClient {

    private PetalsBcSoapMonitoringServiceClient petalsBcSoapMonitoringServiceClient;

    @Override
    protected ComponentMonitoringService createComponentMonitoringService() {
        return new PetalsBcSoapMonitoringService();
    }

    @Override
    protected ComponentMonitoringServiceClient createComponentMonitoringServiceClient(final String componentId)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException {
        this.petalsBcSoapMonitoringServiceClient = jmxClient.getPetalsBcSoapMonitoringServiceClient(componentId);
        return this.petalsBcSoapMonitoringServiceClient;
    }

    /**
     * Check the interface consistency of {@link PetalsBcSoapMonitoringServiceClientImpl#getIncomingWsRequestsCount()}
     */
    @Test
    public void testGetIncomingWsRequestsCount() throws ComponentMonitoringServiceErrorException {
        this.getIncomingWsRequestsCount();
    }

    @Override
    public Map<IncomingWsRequestKey, Long> getIncomingWsRequestsCount() throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getIncomingWsRequestsCount();
    }

    /**
     * Check the interface consistency of
     * {@link PetalsBcSoapMonitoringServiceClientImpl#getInformationHttpRequestsCount()}
     */
    @Test
    public void testGetInformationHttpRequestsCount() throws ComponentMonitoringServiceErrorException {
        this.getInformationHttpRequestsCount();
    }

    @Override
    public Long getInformationHttpRequestsCount() throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getInformationHttpRequestsCount();
    }

    /**
     * Check the interface consistency of {@link PetalsBcSoapMonitoringServiceClientImpl#getUnknownHttpRequestsCount()}
     */
    @Test
    public void testGetUnknownHttpRequestsCount() throws ComponentMonitoringServiceErrorException {
        this.getUnknownHttpRequestsCount();
    }

    @Override
    public Long getUnknownHttpRequestsCount() throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getUnknownHttpRequestsCount();
    }

    @Override
    public Map<String, Long> getServiceContractsHttpRequestsCount() throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getServiceContractsHttpRequestsCount();
    }

    /**
     * Check the interface consistency of
     * {@link PetalsBcSoapMonitoringServiceClientImpl#getIncomingWsRequestsResponseTimes()}
     */
    @Test
    public void testGetIncomingWsRequestsResponseTimes() throws ComponentMonitoringServiceErrorException {
        this.getIncomingWsRequestsResponseTimes();
    }

    @Override
    public Map<IncomingWsRequestKey, Long[]> getIncomingWsRequestsResponseTimes()
            throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getIncomingWsRequestsResponseTimes();
    }

    /**
     * Check the interface consistency of
     * {@link PetalsBcSoapMonitoringServiceClientImpl#getIncomingWsRequestsResponseTimesGroupedBySvcAndExecStatus()}
     */
    @Test
    public void testGetIncomingWsRequestsResponseTimesGroupedBySvcAndExecStatus()
            throws ComponentMonitoringServiceErrorException {
        this.getIncomingWsRequestsResponseTimesGroupedBySvcAndExecStatus();
    }

    @Override
    public Map<IncomingWsRequestKey, Long[]> getIncomingWsRequestsResponseTimesGroupedBySvcAndExecStatus()
            throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getIncomingWsRequestsResponseTimesGroupedBySvcAndExecStatus();
    }

    /**
     * Check the interface consistency of
     * {@link PetalsBcSoapMonitoringServiceClientImpl#getHttpServerThreadPoolMetrics()}
     */
    @Test
    public void testGetHttpServerThreadPoolMetrics() throws ComponentMonitoringServiceErrorException {
        this.getHttpServerThreadPoolMetrics();
    }

    @Override
    public ThreadPoolMetrics getHttpServerThreadPoolMetrics() throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getHttpServerThreadPoolMetrics();
    }

    /**
     * Check the interface consistency of
     * {@link PetalsBcSoapMonitoringServiceClientImpl#getWsClientPoolClientsInUseMax()}
     */
    @Test
    public void testGetWsClientPoolClientsInUseMax() throws ComponentMonitoringServiceErrorException {
        this.getWsClientPoolClientsInUseMax();
    }

    @Override
    public Map<WsClientPoolKey, Long> getWsClientPoolClientsInUseMax() throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getWsClientPoolClientsInUseMax();
    }

    /**
     * Check the interface consistency of
     * {@link PetalsBcSoapMonitoringServiceClientImpl#getWsClientPoolClientsInUseCurrent()}
     */
    @Test
    public void testGetWsClientPoolClientsInUseCurrent() throws ComponentMonitoringServiceErrorException {
        this.getWsClientPoolClientsInUseCurrent();
    }

    @Override
    public Map<WsClientPoolKey, Long> getWsClientPoolClientsInUseCurrent()
            throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getWsClientPoolClientsInUseCurrent();
    }

    /**
     * Check the interface consistency of {@link PetalsBcSoapMonitoringServiceClientImpl#getWsClientPoolExhaustions()}
     */
    @Test
    public void testGetWsClientPoolExhaustions() throws ComponentMonitoringServiceErrorException {
        this.getWsClientPoolExhaustions();
    }

    @Override
    public Map<WsClientPoolKey, Long> getWsClientPoolExhaustions() throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getWsClientPoolExhaustions();
    }

    /**
     * Check the interface consistency of {@link PetalsBcSoapMonitoringServiceClientImpl#getOutgoingWsRequestsCount()}
     */
    @Test
    public void testGetOutgoingWsRequestsCount() throws ComponentMonitoringServiceErrorException {
        this.getOutgoingWsRequestsCount();
    }

    @Override
    public Map<OutgoingWsRequestKey, Long> getOutgoingWsRequestsCount() throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getOutgoingWsRequestsCount();
    }

    /**
     * Check the interface consistency of
     * {@link PetalsBcSoapMonitoringServiceClientImpl#getOutgoingWsRequestsResponseTimes()}
     */
    @Test
    public void testGetOutgoingWsRequestsResponseTimes() throws ComponentMonitoringServiceErrorException {
        this.getOutgoingWsRequestsResponseTimes();
    }

    @Override
    public Map<OutgoingWsRequestKey, Long[]> getOutgoingWsRequestsResponseTimes()
            throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getOutgoingWsRequestsResponseTimes();
    }

    /**
     * Check the interface consistency of
     * {@link PetalsBcSoapMonitoringServiceClientImpl#getOutgoingWsRequestsResponseTimesGroupedBySvcAndExecStatus()}
     */
    @Test
    public void testGetOutgoingWsRequestsResponseTimesGroupedBySvcAndExecStatus()
            throws ComponentMonitoringServiceErrorException {
        this.getOutgoingWsRequestsResponseTimesGroupedBySvcAndExecStatus();
    }

    @Override
    public Map<OutgoingWsRequestKey, Long[]> getOutgoingWsRequestsResponseTimesGroupedBySvcAndExecStatus()
            throws ComponentMonitoringServiceErrorException {
        return this.petalsBcSoapMonitoringServiceClient.getOutgoingWsRequestsResponseTimesGroupedBySvcAndExecStatus();
    }
}