/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.monitoring.container.remote;

import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.IncomingConnectionObjectPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.OutgoingConnectionObjectPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.TcpTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.impl.AbstractServiceClientImpl;
import org.ow2.petals.jmx.api.impl.mbean.monitoring.container.transport.remote.TcpTransporterMonitoringService;
import org.ow2.petals.jmx.api.impl.mbean.monitoring.container.transport.remote.TcpTransporterMonitoringServiceMBean;
import org.ow2.petals.jmx.api.impl.monitoring.container.transport.remote.TcpTransporterMonitoringServiceClientImpl;

/**
 * <p>
 * Unit tests of {@link TcpTransporterMonitoringServiceClientImpl} to check that JMX client interfaces are aligned with
 * the JMX server MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link TcpTransporterMonitoringServiceMBean} to avoid to forget some methods. A unit test
 * will only invoke the associated method of {@link TcpTransporterMonitoringServiceMBean}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class TcpTransporterMonitoringServiceClientImplTest extends AbstractServiceClientImpl
        implements TcpTransporterMonitoringServiceClient {

    private TcpTransporterMonitoringServiceClient tcpTransporterMonitoringServiceClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerTcpTransporterMonitoringService(new TcpTransporterMonitoringService());
        this.tcpTransporterMonitoringServiceClient = this.jmxClient.getTcpTransporterMonitoringServiceClient();
    }

    /**
     * Check the interface consistency of
     * {@link TcpTransporterMonitoringServiceClientImpl#getDelieveredOutgoingMessagesCount()}
     */
    @Test
    public void testGetDelieveredOutgoingMessagesCount() throws TcpTransporterMonitoringServiceErrorException {
        this.getDeliveredOutgoingMessagesCount();
    }

    @Override
    public Map<String[], Long> getDeliveredOutgoingMessagesCount()
            throws TcpTransporterMonitoringServiceErrorException {
        return this.tcpTransporterMonitoringServiceClient.getDeliveredOutgoingMessagesCount();
    }

    /**
     * Check the interface consistency of
     * {@link TcpTransporterMonitoringServiceClientImpl#getDeliveredIncomingMessagesCount()}
     */
    @Test
    public void testGetDeliveredIncomingMessagesCount() throws TcpTransporterMonitoringServiceErrorException {
        this.getDeliveredIncomingMessagesCount();
    }

    @Override
    public Map<String[], Long> getDeliveredIncomingMessagesCount()
            throws TcpTransporterMonitoringServiceErrorException {
        return this.tcpTransporterMonitoringServiceClient.getDeliveredIncomingMessagesCount();
    }

    /**
     * Check the interface consistency of
     * {@link TcpTransporterMonitoringServiceClientImpl#getOutgoingConnectionPoolMetrics()}
     */
    @Test
    public void testGetOutgoingConnectionPoolMetrics() throws TcpTransporterMonitoringServiceErrorException {
        this.getOutgoingConnectionPoolMetrics();
    }

    @Override
    public Map<String, OutgoingConnectionObjectPoolMetrics> getOutgoingConnectionPoolMetrics()
            throws TcpTransporterMonitoringServiceErrorException {
        return this.tcpTransporterMonitoringServiceClient.getOutgoingConnectionPoolMetrics();
    }

    /**
     * Check the interface consistency of
     * {@link TcpTransporterMonitoringServiceClientImpl#getIncomingConnectionPoolMetrics()}
     */
    @Test
    public void testGetIncomingConnectionPoolMetrics() throws TcpTransporterMonitoringServiceErrorException {
        this.getIncomingConnectionPoolMetrics();
    }

    @Override
    public Map<String, IncomingConnectionObjectPoolMetrics> getIncomingConnectionPoolMetrics()
            throws TcpTransporterMonitoringServiceErrorException {
        return this.tcpTransporterMonitoringServiceClient.getIncomingConnectionPoolMetrics();
    }

}
